<?php

namespace qfq;

//use qfq;

require_once(__DIR__ . '/../Constants.php');
require_once(__DIR__ . '/../database/Database.php');
require_once(__DIR__ . '/../store/Store.php');


class Sendmail {

    /**
     * Sends a mail as specified in $mailarr.
     * If there is no receiver specified as 'TO': no mail is sent. This is ok and no error.
     * Logs every send mail as a record in table `mailLog`. Additionally a `grId` and a `xId` can be specified
     * to assign the log entry to a specific action.
     * The log record also contains some information which generates the mail (form/formelement or QFQ query).
     *
     * Structure mailarr:
     *    SENDMAIL_IDX_RECEIVER             email address(es)
     *    SENDMAIL_IDX_SENDER               email address
     *    SENDMAIL_IDX_SUBJECT              string
     *    SENDMAIL_IDX_BODY                 string
     *    SENDMAIL_IDX_REPLY_TO             optional: email address
     *    SENDMAIL_IDX_FLAG_AUTO_SUBMIT     optional: 'on'|'off', default: 'on'
     *    SENDMAIL_IDX_GR_ID                optional: integer
     *    SENDMAIL_IDX_X_ID                 optional: integer
     *
     * @param $mailarr
     *
     * @throws UserFormException
     */
    public function __construct(array $mailarr) {

        $addBody = '';

        // If there is no 'Receiver': do not send a mail.
        if (!isset($mailarr[SENDMAIL_IDX_RECEIVER]) || $mailarr[SENDMAIL_IDX_RECEIVER] === '') {
            return;
        }

        if (count($mailarr) < 4 || $mailarr[SENDMAIL_IDX_SENDER] === '' || $mailarr[SENDMAIL_IDX_SUBJECT] === '' || $mailarr[SENDMAIL_IDX_BODY] === '') {
            throw new UserFormException("Error sendmail missing one of: receiver, sender, subject or body", ERROR_SENDMAIL_MISSING_VALUE);
        }

        $store = Store::getInstance('');
        $redirectAllMail = $store->getVar(SYSTEM_REDIRECT_ALL_MAIL_TO, STORE_SYSTEM);
        if ($redirectAllMail !== false) {
            $addBody .= "All QFQ outgoing mails are catched and redirected to you." . PHP_EOL . "Original receiver:" . PHP_EOL;
            $addBody .= 'TO: ' . $mailarr[SENDMAIL_IDX_RECEIVER] . PHP_EOL;
            $addBody .= 'CC: ' . $mailarr[SENDMAIL_IDX_RECEIVER_CC] . PHP_EOL;
            $addBody .= 'BCC: ' . $mailarr[SENDMAIL_IDX_RECEIVER_BCC] . PHP_EOL;
            $addBody .= PHP_EOL . "==========================================" . PHP_EOL . PHP_EOL;

            $mailarr[SENDMAIL_IDX_RECEIVER] = $redirectAllMail;
            $mailarr[SENDMAIL_IDX_RECEIVER_CC] = '';
            $mailarr[SENDMAIL_IDX_RECEIVER_BCC] = '';
        }

        $header = $this->buildHeader($mailarr);

        if (!(mb_send_mail($mailarr[SENDMAIL_IDX_RECEIVER], $mailarr[SENDMAIL_IDX_SUBJECT], $addBody . $mailarr[SENDMAIL_IDX_BODY], $header, "-f " . $mailarr[SENDMAIL_IDX_SENDER]))) {
            throw new UserFormException("Error sendmail failed.", ERROR_SENDMAIL);
        }

        $this->mailLog($mailarr, $header);
    }

    /**
     * @param array $mailarr
     *
     * @return string
     */
    private function buildHeader(array $mailarr) {

        // "\r\n" needs to be enclosed in double ticks to correctly converted to 0x0d 0x0a,
        $header = "From: " . $mailarr[SENDMAIL_IDX_SENDER] . "\r\n";

        if (isset($mailarr[SENDMAIL_IDX_REPLY_TO]) && $mailarr[SENDMAIL_IDX_REPLY_TO] != '') {
            $header .= "Reply-To: " . $mailarr[SENDMAIL_IDX_REPLY_TO] . "\r\n";
        }

        // By default 'on'
        if (!isset($mailarr[SENDMAIL_IDX_FLAG_AUTO_SUBMIT]) || $mailarr[SENDMAIL_IDX_FLAG_AUTO_SUBMIT] === 'on') {
            $header .= "Auto-Submitted: auto-send\r\n";
        }

        if (isset($mailarr[SENDMAIL_IDX_RECEIVER_CC]) && $mailarr[SENDMAIL_IDX_RECEIVER_CC] != '') {
            $header .= "Cc: " . $mailarr[SENDMAIL_IDX_RECEIVER_CC] . "\r\n";
        }

        if (isset($mailarr[SENDMAIL_IDX_RECEIVER_BCC]) && $mailarr[SENDMAIL_IDX_RECEIVER_BCC] != '') {
            $header .= "Bcc: " . $mailarr[SENDMAIL_IDX_RECEIVER_BCC] . "\r\n";
        }

        return $header;
    }

    /**
     * Creates a new MailLog Record based on $mailArr / $header.
     *
     * @param array $mailarr
     * @param       $header
     *
     * @throws CodeException
     * @throws DbException
     */
    private function mailLog(array $mailarr, $header) {

        $log = array();

        // Log
        $log[SENDMAIL_IDX_RECEIVER] = $mailarr[SENDMAIL_IDX_RECEIVER];
        $log[SENDMAIL_IDX_SENDER] = $mailarr[SENDMAIL_IDX_SENDER];
        $log[SENDMAIL_IDX_SUBJECT] = $mailarr[SENDMAIL_IDX_SUBJECT];
        $log[SENDMAIL_IDX_BODY] = $mailarr[SENDMAIL_IDX_BODY];
        $log[4] = $header;
        $log[5] = empty($mailarr[SENDMAIL_IDX_GR_ID]) ? 0 : $mailarr[SENDMAIL_IDX_GR_ID];
        $log[6] = empty($mailarr[SENDMAIL_IDX_X_ID]) ? 0 : $mailarr[SENDMAIL_IDX_X_ID];
        $log[7] = empty($mailarr[SENDMAIL_IDX_SRC]) ? 0 : $mailarr[SENDMAIL_IDX_SRC];

        $db = new Database();
        $db->sql('INSERT INTO MailLog (`receiver`, `sender`, `subject`, `body`, `header`, `grId`, `xId`, `src`, `modified`, `created`) VALUES ( ?, ? ,?, ?, ? ,?, ?, ?, NOW(), NOW() )', ROW_REGULAR, $log);

    }
}
